<?php

/**
 * WC_Gateway_YottaPay class.
 *
 * @extends WC_Payment_Gateway
 */
class WC_Gateway_YottaPay extends WC_Payment_Gateway
{
    /**
     * Constructor for the gateway.
     */
    public function __construct()
    {
        // Setup general properties
        $this->setup_properties();

        // Load the settings
        $this->init_form_fields();
        $this->init_settings();

        // Get settings
        $this->title = 'Online Banking UK - Faster Checkout &#8482; | Yotta Pay &#174;';
        $this->description = sprintf( __( 'Bank transfers made easy.<br>One-click payment with your UK mobile banking. No card details required.<br>Connect to your bank and complete your purchase instantly!<br>Powered by Yotta Pay &#174;.', 'yottapay-payment-gateway' ));
        $this->sandbox = $this->get_option('sandbox');
        $this->yottapay_merchant_identifier = $this->get_option('yottapay_merchant_identifier');
        $this->yottapay_payment_key = $this->get_option('yottapay_payment_key');

        //Update options
        add_action('woocommerce_update_options_payment_gateways_' . $this->id, array($this, 'process_admin_options'));

        // Yotta Pay API process payment result
        add_action('woocommerce_api_yottapay_process_payment_result', array($this, 'yottapay_process_payment_result'));

        // Yotta Pay API refund request
        add_action('woocommerce_api_yottapay_process_refund', array($this, 'yottapay_process_refund'));

        // Yotta Pay API refund result
        add_action('woocommerce_api_yottapay_process_refund_result', array($this, 'yottapay_process_refund_result'));

        // Add refund form endpoint
        add_action('woocommerce_account_yottapay-refund-request-form_endpoint', array($this, 'yottapay_refund_request_form'));

        //Add script on checkout
        add_action('woocommerce_review_order_before_payment', array($this, 'yottapay_add_script'));
    }

    /**
     * Setup general properties for the gateway.
     */
    protected function setup_properties()
    {
        $this->id = 'yottapay';
        $this->icon = apply_filters('woocommerce_yottapay_icon', plugins_url('../assets/icon.png', __FILE__));
        $this->method_title = __('Yotta Pay', 'yottapay-payment-gateway');
        $this->yottapay_merchant_identifier = __('Add your Yotta Pay Merchant Identifier', 'yottapay-payment-gateway');
        $this->yottapay_payment_key = __('Add your Yotta Pay Payment Key', 'yottapay-payment-gateway');
        $this->method_description = sprintf(__('Yotta Pay is the fastest and most secure way to accept payments via UK bank account. <br><br>More information about connecting Yotta Pay API for online stores, applications, and services <a href="%s">here</a>.', 'yottapay-payment-gateway'), 'https://www.yottapay.co.uk/online-checkout-guideline');
        $this->has_fields = false;
    }

    /**
     * Initialise Gateway Settings Form Fields.
     */
    public function init_form_fields()
    {
        $this->form_fields = array(
            'enabled' => array(
                'title' => __('Enable / Disable', 'yottapay-payment-gateway'),
                'label' => __('Enable Yotta Pay', 'yottapay-payment-gateway'),
                'type' => 'checkbox',
                'description' => '',
                'default' => 'no',
            ),
            'sandbox' => array(
                'title' => __('Sandbox mode', 'yottapay-payment-gateway'),
                'label' => __('Enable Sandbox Mode', 'yottapay-payment-gateway'),
                'type' => 'checkbox',
                'description' => '',
                'default' => 'yes',
            ),
            'yottapay_merchant_identifier' => array(
                'title' => __('Merchant Identifier', 'yottapay-payment-gateway'),
                'type' => 'text',
                'description' => __('Add your Yotta Pay Merchant Identifier', 'yottapay-payment-gateway'),
                'desc_tip' => true,
            ),
            'yottapay_payment_key' => array(
                'title' => __('Payment Key', 'yottapay-payment-gateway'),
                'type' => 'password',
                'description' => __('Add your Yotta Pay Payment Key', 'yottapay-payment-gateway'),
                'desc_tip' => true,
            ),
        );
    }

    /**
     * Process payment.
     */
    public function process_payment($order_id)
    {
        return WC_YottaPay_Payment::process_payment($this, $order_id);
    }

    /**
     * Yotta Pay API payment callback.
     */
    public function yottapay_process_payment_result()
    {
        WC_YottaPay_Payment::process_payment_result($this);
    }

    /**
     * Create refund request to Yotta Pay API and return the result status.
     *
     * @since 1.2.0
     */
    public function yottapay_process_refund()
    {
        WC_YottaPay_Refund::process_refund($this);
    }

    /**
     * Yotta Pay API refund callback.
     *
     * @since 1.2.0
     */
    public function yottapay_process_refund_result()
    {
        WC_YottaPay_Refund::process_refund_result($this);
    }

    /**
     * Show refund request form.
     *
     * @since 1.2.0
     */
    public function yottapay_refund_request_form()
    {
        require_once YOTTAPAY_TEMPLATE_PATH . '/refund-request.php';
    }

    /**
     * Reload checkout page to disable incorrect blockUI.
     */
    public function yottapay_add_script()
    {
        WC_YottaPay_Payment::add_script();
    }
}
