<?php

/**
 * WC_YottaPay_Payment class.
 *
 * @since 1.2.0
 */
class WC_YottaPay_Payment
{
    const YOTTAPAY_API_PAYMENT_ENDPOINT = '/launcher/shop/paymentgateway/new';

    /**
     * Create payment intent request to Yotta Pay API and return the result with redirect url.
     */
    public static function process_payment($gateway, $order_id)
    {
        try {
            WC_YottaPay_Common::get_logger()->info('==== Process payment started for order #' . $order_id . ' ====', ['source' => WC_YottaPay_Common::WC_LOG_FILENAME]);

            $order = wc_get_order($order_id);

            // This will throw exception if not valid.
            self::validate_billing_country($order);
            self::validate_order_currency($order);
            self::validate_minimum_order_amount($order);

            //Create payment intent request to Yotta Pay API
            $request_body = json_encode(self::get_data_to_send($gateway, $order));
            $response = wp_remote_post(WC_YottaPay_Common::get_yottapay_api_url($gateway->sandbox) . self::YOTTAPAY_API_PAYMENT_ENDPOINT, array(
                'timeout' => 20,
                'headers' => array(
                    'Content-Type' => 'application/json'
                ),
                'body' => $request_body
            ));

            $response_data = WC_YottaPay_Common::validate_response_status($response, $request_body);

            if (!array_key_exists('url_process_payment_intent', $response_data)) {
                throw new Exception('Missing required parameters in Yotta Pay API response.');
            }

            WC_YottaPay_Common::get_logger()->info('Payment intent has been created.', ['source' => WC_YottaPay_Common::WC_LOG_FILENAME]);

            return [
                'result' => 'success',
                'redirect' => sanitize_text_field($response_data['url_process_payment_intent']),
            ];
        } catch (YottaPayException $e) {
            wc_add_notice($e->getMessage(), 'error');

            return [
                'result' => 'fail',
                'redirect' => '',
            ];
        } catch (Exception $e) {
            if (isset($response_data)) {
                WC_YottaPay_Common::get_logger()->warning('response_data: ' . print_r($response_data, TRUE), ['source' => WC_YottaPay_Common::WC_LOG_FILENAME]);
            }

            WC_YottaPay_Common::get_logger()->warning($e->getMessage(), ['source' => WC_YottaPay_Common::WC_LOG_FILENAME]);

            wc_add_notice('Payment was failed. Please contact the store owner to solve the problem.', 'error');

            return [
                'result' => 'fail',
                'redirect' => '',
            ];
        }
    }

    /**
     * Yotta Pay API Callback.
     * Change processing order status.
     */
    public static function process_payment_result($gateway)
    {
        try {
            WC_YottaPay_Common::get_logger()->info('==== Process payment result started ====', ['source' => WC_YottaPay_Common::WC_LOG_FILENAME]);

            $request_data = WC_YottaPay_Common::get_request_data();

            $order = self::check_request_data($gateway, $request_data);

            self::check_request_sign($gateway, $request_data, $order);

            self::check_response_code($request_data, $order);

            WC_YottaPay_Common::get_logger()->info('Process payment result has been finished for order #' . $order->get_id(), ['source' => WC_YottaPay_Common::WC_LOG_FILENAME]);
        } catch (Exception $e) {
            if (isset($order)) {
                WC_YottaPay_Common::get_logger()->warning('Process payment result has been failed for order #' . $order->get_id(), ['source' => WC_YottaPay_Common::WC_LOG_FILENAME]);
            }
            if (isset($request_data)) {
                WC_YottaPay_Common::get_logger()->warning('request_data: ' . print_r($request_data, TRUE), ['source' => WC_YottaPay_Common::WC_LOG_FILENAME]);
            }
            WC_YottaPay_Common::get_logger()->warning($e->getMessage(), ['source' => WC_YottaPay_Common::WC_LOG_FILENAME]);
        }
    }

    /**
     * Validate order currency.
     */
    private static function validate_order_currency($order)
    {
        if ($order->get_currency() != 'GBP') {
            throw new YottaPayException('Sorry, Yotta Pay currently works only with GBP.');
        }
    }

    /**
     * Validate billing country.
     */
    private static function validate_billing_country($order)
    {
        if ($order->get_billing_country() != 'GB') {
            throw new YottaPayException('Sorry, Yotta Pay currently is only available in the United Kingdom.');
        }
    }

    /**
     * Validate that the order meets the minimum order amount.
     */
    private static function validate_minimum_order_amount($order)
    {
        if (number_format(round($order->get_total(), 2, PHP_ROUND_HALF_UP), 2, '.', '') < 0.01) {
            throw new YottaPayException('Sorry, the minimum allowed order total is 0.01GBP to use Yotta Pay payment method.');
        }
    }

    /**
     * Convert order data to spicial format and calculate signature.
     */
    private static function get_data_to_send($gateway, $order)
    {
        $body = array(
            'type' => 'creation',
            'shop_transaction_identifier' => 'WOO#' . $order->get_customer_id() . '-' . $order->get_id(),
            'merchant_identifier' => trim($gateway->yottapay_merchant_identifier),
            'customer_identifier' => $order->get_customer_id(),
            'amount' => number_format(round($order->get_total(), 2, PHP_ROUND_HALF_UP), 2, '.', ''),
            'currency' => $order->get_currency(),
            'url_payment_result' => get_site_url() . '/wc-api/yottapay_process_payment_result',
            'url_merchant_page_success' => get_site_url() . '/checkout/order-received/' . $order->get_id() . '?key=' . $order->get_order_key(),
            'url_merchant_page_cancel' => get_site_url() . '/checkout/order-received/' . $order->get_id() . '?key=' . $order->get_order_key(),
            'yotta_notification_id' => '',
            'signature' => ''
        );

        $dataToSign = (
            $body['shop_transaction_identifier']
            . $body['merchant_identifier']
            . $body['customer_identifier']
            . $body['amount']
            . $body['currency']
            . $body['url_payment_result']
            . $body['url_merchant_page_success']
            . $body['url_merchant_page_cancel']
            . trim($gateway->yottapay_payment_key)
        );

        $sign = hash('sha256', utf8_encode($dataToSign));

        $body['signature'] = $sign;

        return $body;
    }

    /**
     * Check required fields, then return the appropriate order.
     */
    private static function check_request_data($gateway, $request_data)
    {
        if (!array_key_exists('type', $request_data) ||
            !array_key_exists('yottapay_transaction_identifier', $request_data) ||
            !array_key_exists('shop_transaction_identifier', $request_data) ||
            !array_key_exists('merchant_identifier', $request_data) ||
            !array_key_exists('customer_identifier', $request_data) ||
            !array_key_exists('amount', $request_data) ||
            !array_key_exists('currency', $request_data) ||
            !array_key_exists('response_code', $request_data) ||
            !array_key_exists('signature', $request_data)) {
            throw new Exception('Missing required parameters in request.');
        }

        $splited_shop_transaction_identifier = explode('-', $request_data['shop_transaction_identifier']);
        $order_id = end($splited_shop_transaction_identifier);
        $order = wc_get_order($order_id);

        if (!$order) {
            throw new Exception('No order.');
        }

        $order_status = $order->get_status();

        if ($order_status != 'pending' && $order_status != 'failed' && $order_status != 'cancelled') {
            throw new Exception('Invalid order status for processing: ' . $order_status . '.');
        }

        //Match source data
        $source_customer_identifier = $order->get_customer_id();
        $source_merchant_identifier = trim($gateway->yottapay_merchant_identifier);
        $source_shop_transaction_identifier = ('WOO#' . $source_customer_identifier . '-' . $order->get_id());
        $source_amount = number_format(round($order->get_total(), 2, PHP_ROUND_HALF_UP), 2, '.', '');
        $source_currency = $order->get_currency();

        if (sanitize_text_field($request_data['type']) != 'statuschange' ||
            sanitize_text_field($request_data['shop_transaction_identifier']) != $source_shop_transaction_identifier ||
            sanitize_text_field($request_data['merchant_identifier']) != $source_merchant_identifier ||
            sanitize_text_field($request_data['customer_identifier']) != $source_customer_identifier ||
            sanitize_text_field($request_data['amount']) != $source_amount ||
            sanitize_text_field($request_data['currency']) != $source_currency) {
            throw new Exception('Invalid request data.');
        }

        return $order;
    }

    /**
     * Caluclate signature and compare with requested value.
     */
    private static function check_request_sign($gateway, $request_data, $order)
    {
        $dataToSign = (
            sanitize_text_field($request_data['yottapay_transaction_identifier'])
            . sanitize_text_field($request_data['shop_transaction_identifier'])
            . sanitize_text_field($request_data['merchant_identifier'])
            . sanitize_text_field($request_data['customer_identifier'])
            . sanitize_text_field($request_data['amount'])
            . sanitize_text_field($request_data['currency'])
            . sanitize_text_field($request_data['response_code'])
            . trim($gateway->yottapay_payment_key)
        );

        $sign = hash('sha256', utf8_encode($dataToSign));

        if (sanitize_text_field($request_data['signature']) != $sign) {
            throw new Exception('Signature is wrong (' . sanitize_text_field($request_data['yottapay_transaction_identifier']) . ').');
        }
    }

    /**
     * Check operation result code from request and update order status.
     */
    private static function check_response_code($request_data, $order)
    {
        WC_YottaPay_Common::get_logger()->info('Response code: ' . $request_data['response_code'], ['source' => WC_YottaPay_Common::WC_LOG_FILENAME]);

        if ($request_data['response_code'] == '0') {
            $order->set_transaction_id(sanitize_text_field($request_data['yottapay_transaction_identifier']));
            $order->update_status('processing');
        } elseif ($request_data['response_code'] == '2') {
            $order->update_status('cancelled');
        } else {
            $order->update_status('failed');
        }
    }

    /**
     * Reload checkout page to disable incorrect blockUI.
     *
     * @since 1.1.0
     */
    public static function add_script()
    {
        // jQuery code
        ?>
        <script type="text/javascript">
            (function ($) {
                var reloadTimeoutId = -1;

                function yottapayReloadCheckoutPage() {
                    if (reloadTimeoutId && (reloadTimeoutId != -1)) {
                        clearTimeout(reloadTimeoutId);
                        reloadTimeoutId = -1;
                        location.reload(true);
                    }
                }

                var checkout_form = $('form.checkout');
                checkout_form.on('checkout_place_order', function () {
                    try {
                        var checkedYottaPayPaymentMethod = $('form[name="checkout"]').find(':input[id="payment_method_yottapay"]:checked');
                        if (checkedYottaPayPaymentMethod.length) {
                            reloadTimeoutId = window.setTimeout(yottapayReloadCheckoutPage, 20000);
                        }
                    } catch (e) {
                        console.error(e);
                    } finally {
                        return true;
                    }
                });
                $(document.body).on('checkout_error', function () {
                    if ($('.woocommerce-error li').length) {
                        if (reloadTimeoutId && (reloadTimeoutId != -1)) {
                            clearTimeout(reloadTimeoutId);
                            reloadTimeoutId = -1;
                        }
                    }
                });
                window.addEventListener("pageshow", function (event) {
                    var historyTraversal = event.persisted || (typeof window.performance != "undefined" && window.performance.navigation.type === 2);
                    if (historyTraversal) {
                        if (reloadTimeoutId && (reloadTimeoutId != -1)) {
                            clearTimeout(reloadTimeoutId);
                            reloadTimeoutId = -1;
                        }
                        location.reload(true);
                    }
                });
            })(jQuery);
        </script>
        <?php
    }
}
