<?php

/**
 * WC_YottaPay_Refund class.
 *
 * @since 1.2.0
 */
class WC_YottaPay_Refund
{
    const YOTTAPAY_API_REFUND_ENDPOINT = '/launcher/shop/paymentgateway/refund';

    /**
     * Create refund request to Yotta Pay API and return the result status.
     */
    public static function process_refund($gateway)
    {
        try {
            //Validate request data
            $validated_data = self::validate_refund_request_data();
            $order = $validated_data['order'];

            WC_YottaPay_Common::get_logger()->info('==== Process refund started for order #' . $order->get_id() . ' ====', ['source' => WC_YottaPay_Common::WC_LOG_FILENAME]);

            //Validate available length
            self::validate_available_length($validated_data);

            //Validate Account number and Sort code fields
            try {
                self::validate_numeric($validated_data);            
            } catch (Exception $e) {            
                WC_YottaPay_Common::get_logger()->warning($e->getMessage(), ['source' => WC_YottaPay_Common::WC_LOG_FILENAME]);

                $result = array(
                    'status' => '0',
                    'error' => 'Incorrect Account number or Sort code'
                );               

                header('Content-Type: application/json; charset=utf-8');
                echo json_encode($result);

                die();
            }           

            //Create refund request
            $request_body = json_encode(self::get_data_to_send($gateway, $order, $validated_data));
            $response = wp_remote_post(WC_YottaPay_Common::get_yottapay_api_url($gateway->sandbox) . self::YOTTAPAY_API_REFUND_ENDPOINT, array(
                'timeout' => 20,
                'headers' => array(
                    'Content-Type' => 'application/json'
                ),
                'body' => $request_body
            ));

            //Validate response
            $response_data = WC_YottaPay_Common::validate_response_status($response, $request_body);

            if (!array_key_exists('ok', $response_data)) {
                throw new Exception('Missing required parameters in Yotta Pay API response.');
            }

            //Validate ok status
            $respose_status = sanitize_text_field($response_data['ok']);
            if (!(boolval($respose_status) && wc_strtolower($respose_status) != 'false')) {
                throw new Exception('Request has been failed.');
            }

            //Set notice and return result
            wc_add_notice('Request has been submitted successfully', 'success');

            $result = array(
                'status' => '1',
                'error' => ''
            );

            WC_YottaPay_Common::get_logger()->info('Refund intent has been created.', ['source' => WC_YottaPay_Common::WC_LOG_FILENAME]);

            header('Content-Type: application/json; charset=utf-8');
            echo json_encode($result);

            die();
        } catch (Exception $e) {
            if (isset($order)) {
                WC_YottaPay_Common::get_logger()->warning('Refund processing exception.', ['source' => WC_YottaPay_Common::WC_LOG_FILENAME]);
            }

            WC_YottaPay_Common::get_logger()->warning($e->getMessage(), ['source' => WC_YottaPay_Common::WC_LOG_FILENAME]);

            $result = array(
                'status' => '0',
                'error' => 'Refund request has been failed. Please contact the store owner to solve the problem.'
            );

            if (isset($response_data)) {
                $result['error'] = $response_data['error_message'];
                WC_YottaPay_Common::get_logger()->warning('response_data: ' . print_r($response_data, TRUE), ['source' => WC_YottaPay_Common::WC_LOG_FILENAME]);
            }

            header('Content-Type: application/json; charset=utf-8');
            echo json_encode($result);

            die();
        }
    }

    /**
     * Yotta Pay API refund callback.
     */
    public static function process_refund_result($gateway)
    {
        try {
            WC_YottaPay_Common::get_logger()->info('==== Process refund result started ====', ['source' => WC_YottaPay_Common::WC_LOG_FILENAME]);

            $request_data = WC_YottaPay_Common::get_request_data();

            $order = self::check_request_data($gateway, $request_data);

            self::check_request_sign($gateway, $request_data, $order);

            $order->update_status('refunded');

            WC_YottaPay_Common::get_logger()->info('Process refund result has been finished for order #' . $order->get_id(), ['source' => WC_YottaPay_Common::WC_LOG_FILENAME]);
        } catch (Exception $e) {
            if (isset($order)) {
                WC_YottaPay_Common::get_logger()->warning('Process refund result has been failed for order #' . $order->get_id(), ['source' => WC_YottaPay_Common::WC_LOG_FILENAME]);
            }
            if (isset($request_data)) {
                WC_YottaPay_Common::get_logger()->warning('refund_result request_data: ' . print_r($request_data, TRUE), ['source' => WC_YottaPay_Common::WC_LOG_FILENAME]);
            }
            WC_YottaPay_Common::get_logger()->warning($e->getMessage(), ['source' => WC_YottaPay_Common::WC_LOG_FILENAME]);
        }
    }

    /**
     * Validate refund form data.
     */
    private static function validate_refund_request_data()
    {
        $postdata = wp_unslash($_POST);

        //Verify nonce
        if (!isset($postdata['yottapay_refund_request_nonce'])) {
            throw new Exception('Missing required parameters.');
        }
        if (!wp_verify_nonce($postdata['yottapay_refund_request_nonce'], 'yottapay_refund_request_form_nonce_action')) {
            throw new Exception('Verify nonce exception.');
        }

        //Verify required parameters
        if (!isset($postdata['order_id']) ||
            !isset($postdata['refund_receiver_full_name']) ||
            !isset($postdata['refund_receiver_account_number']) ||
            !isset($postdata['refund_receiver_sort_code']) ||
            !isset($postdata['refund_receiver_comment'])) {
            throw new Exception('Missing required parameters.');
        }

        //Verify order
        $order_id = sanitize_text_field($postdata['order_id']);
        $order = wc_get_order($order_id);
        if (!$order) {
            throw new Exception('No order.');
        }

        //Verify order status
        $order_status = $order->get_status();
        if ($order_status != 'processing') {
            throw new Exception('Invalid order status for refund: ' . $order_status);
        }

        //Verify customer
        if ($order->get_customer_id() != get_current_user_id()  ||
            get_current_user_id() == '0'
            || !is_user_logged_in()) {
            throw new Exception('Another customer.');
        }

        $sanitized_data = array(
            'order' => $order,
            'refund_receiver_full_name' => trim(sanitize_text_field($postdata['refund_receiver_full_name'])),
            'refund_receiver_account_number' => trim(sanitize_text_field($postdata['refund_receiver_account_number'])),
            'refund_receiver_sort_code' => trim(sanitize_text_field($postdata['refund_receiver_sort_code'])),
            'refund_receiver_comment' => trim(sanitize_text_field($postdata['refund_receiver_comment']))
        );

        return $sanitized_data;
    }

    /**
     * Validate available length of input fields.
     */
    private static function validate_available_length($validated_data)
    {
        if (strlen($validated_data['refund_receiver_full_name']) == 0 ||
            strlen($validated_data['refund_receiver_full_name']) > 64) {
            throw new Exception('Account holder name is required field (length up to 64 characters).');
        }
        if (strlen($validated_data['refund_receiver_account_number']) != 8) {
            throw new Exception('Account number is required field (length 8 characters).');
        }
        if (strlen($validated_data['refund_receiver_sort_code']) != 6) {
            throw new Exception('Sort code is required field (length 6 characters).');
        }
        if (strlen($validated_data['refund_receiver_comment']) == 0 ||
            strlen($validated_data['refund_receiver_comment']) > 1000) {
            throw new Exception('Comment is required field (length up to 1000 characters).');
        }
    }

    /**
     * Validate values of Account number and Sort code fields
     */
    private static function validate_numeric($validated_data)
    {
        $accountNumberCleared = preg_replace( '/[^0-9]/', '', $validated_data['refund_receiver_account_number'] );
        $sortCodeCleared = preg_replace( '/[^0-9]/', '', $validated_data['refund_receiver_sort_code'] );
        
        if (strlen($accountNumberCleared) != 8) {
            throw new Exception('Invalid account number.');
        }
        if (strlen($sortCodeCleared) != 6) {
            throw new Exception('Invalid sort code.');
        }
    }

    /**
     * Convert order data to spicial format and calculate signature.
     */
    private static function get_data_to_send($gateway, $order, $validated_data)
    {
        $body = array(
            'type' => 'refund',
            'yottapay_transaction_identifier' => $order->get_transaction_id(),
            'merchant_identifier' => trim($gateway->yottapay_merchant_identifier),
            'url_payment_refund_result' => get_site_url() . '/wc-api/yottapay_process_refund_result',
            'refund_receiver_full_name' => $validated_data['refund_receiver_full_name'],
            'refund_receiver_account_number' => $validated_data['refund_receiver_account_number'],
            'refund_receiver_sort_code' => $validated_data['refund_receiver_sort_code'],
            'refund_receiver_comment' => $validated_data['refund_receiver_comment'],
            'signature' => ''
        );

        $dataToSign = (
            $body['merchant_identifier']
            . $body['url_payment_refund_result']
            . $body['yottapay_transaction_identifier']
            . $body['refund_receiver_full_name']
            . $body['refund_receiver_account_number']
            . $body['refund_receiver_sort_code']
            . $body['refund_receiver_comment']
            . trim($gateway->yottapay_payment_key)
        );

        $sign = hash('sha256', utf8_encode($dataToSign));

        $body['signature'] = $sign;

        return $body;
    }

    /**
     * Check required fields, then return the appropriate order.
     */
    private static function check_request_data($gateway, $request_data)
    {
        if (!array_key_exists('type', $request_data) ||
            !array_key_exists('customer_identifier', $request_data) ||
            !array_key_exists('merchant_identifier', $request_data) ||
            !array_key_exists('shop_transaction_identifier', $request_data) ||
            !array_key_exists('yottapay_transaction_identifier', $request_data) ||
            !array_key_exists('signature', $request_data)) {
            throw new Exception('Missing required parameters in request.');
        }

        $splited_shop_transaction_identifier = explode('-', $request_data['shop_transaction_identifier']);
        $order_id = end($splited_shop_transaction_identifier);
        $order = wc_get_order($order_id);

        if (!$order) {
            throw new Exception('No order.');
        }

        $order_status = $order->get_status();

        if ($order_status != 'processing' && $order_status != 'completed') {
            throw new Exception('Invalid order status for refund: ' . $order_status);
        }

        //Match source data
        if(sanitize_text_field($request_data['customer_identifier'])  == '0') {
            $source_customer_identifier = '0'; //ordered from guest
        } else {
            $source_customer_identifier = $order->get_customer_id();
        }
        $source_merchant_identifier = trim($gateway->yottapay_merchant_identifier);
        $source_shop_transaction_identifier = ('WOO#' . $source_customer_identifier . '-' . $order->get_id());
        $source_yottapay_transaction_identifier = $order->get_transaction_id();

        if (sanitize_text_field($request_data['type']) != 'refundcompleted' ||
            sanitize_text_field($request_data['shop_transaction_identifier']) != $source_shop_transaction_identifier ||
            sanitize_text_field($request_data['yottapay_transaction_identifier']) != $source_yottapay_transaction_identifier ||
            sanitize_text_field($request_data['merchant_identifier']) != $source_merchant_identifier ||
            sanitize_text_field($request_data['customer_identifier']) != $source_customer_identifier) {
            throw new Exception('Invalid request data.');
        }

        return $order;
    }

    /**
     * Caluclate signature and compare with requested value.
     */
    private static function check_request_sign($gateway, $request_data, $order)
    {
        $dataToSign = (
            sanitize_text_field($request_data['yottapay_transaction_identifier'])
            . sanitize_text_field($request_data['shop_transaction_identifier'])
            . sanitize_text_field($request_data['merchant_identifier'])
            . sanitize_text_field($request_data['customer_identifier'])
            . trim($gateway->yottapay_payment_key)
        );

        $sign = hash('sha256', utf8_encode($dataToSign));

        if (sanitize_text_field($request_data['signature']) != $sign) {
            throw new Exception('check_request_sign: signature is wrong (' . sanitize_text_field($request_data['yottapay_transaction_identifier']) . ').');
        }
    }

    /**
     * Yotta Pay refund form endpoint.
     */
    public static function add_myaccount_refund_endpoint($value = '')
    {
        add_rewrite_endpoint('yottapay-refund-request-form', EP_ROOT | EP_PAGES);
        flush_rewrite_rules();
    }

    /**
     * Yotta Pay refund button.
     */
    public static function add_refund_btn_to_actions($actions, $order)
    {
        if (!$order) {
            return $actions;
        }

        $order_status = $order->get_status();
        if ($order_status != 'processing' && $order_status != 'completed') {
            return $actions;
        }

        $order_paymentmethod = $order->get_payment_method();
        if ($order_paymentmethod != 'yottapay') {
            return $actions;
        }

        //Verify customer
        if (get_current_user_id() == '0' || !is_user_logged_in()) {
            return false;
        }

        $url = esc_url_raw(wc_get_account_endpoint_url('yottapay-refund-request-form') . $order->get_id());

        $actions['yottapay_process_refund'] = array('url' => $url, 'name' => 'Yotta Pay refund');
        return $actions;
    }

    /**
     * Yotta Pay refund button.
     */
    public static function add_refund_btn_to_order($order)
    {
        if (!$order) {
            return false;
        }

        $order_status = $order->get_status();
        if ($order_status != 'processing' && $order_status != 'completed') {
            return false;
        }

        $order_paymentmethod = $order->get_payment_method();
        if ($order_paymentmethod != 'yottapay') {
            return false;
        }

        //Verify customer
        if (get_current_user_id() == '0' || !is_user_logged_in()) {
            return false;
        }

        $url = esc_url_raw(wc_get_account_endpoint_url('yottapay-refund-request-form') . $order->get_id());

        ?>
        <p>
            <a href="<?php echo esc_url($url); ?>" class="button"><?php echo esc_html('Yotta Pay refund'); ?></a>
        </p>
        <?php
    }

    /**
     * Disable admin refund button.
     */
    public static function disable_admin_refund_button($actions, $order)
    {
        try {
            if (!$order) {
                return $actions;
            }

            $order_paymentmethod = $order->get_payment_method();
            if ($order_paymentmethod != 'yottapay') {
                return $actions;
            }

            $order_status = $order->get_status();

            if ($order_status != 'refunded') {
                echo '<style>                
                button.refund-items {
                    pointer-events: none !important;
                    border: none !important;
                }
                </style>';
                // jQuery code
                ?>
                <script type="text/javascript">
                    (function ($) {
                        $(document).ready(function() {
                            var refundButton = $(".button.refund-items");
                            refundButton.replaceWith("<?php echo sprintf( __( '<br><br><p style=\"text-align: left;\">To make Yotta Pay refund you can use <a href=\"https://www.yottapay.co.uk/faq-questions/what-is-dashboard\">Yotta Pay Dashboard</a> tool or ask the customer to request from view-order page if the purchase was made by a client through their account with your site.</p>', 'yottapay-payment-gateway' )); ?>");
                        });                
                    })(jQuery);
                </script>
                <?php
            } else {
                echo '<style>                
                button.refund-items {
                    pointer-events: none !important;
                    border: none !important;
                }
                </style>';
                // jQuery code
                ?>
                <script type="text/javascript">
                    (function ($) {
                        $(document).ready(function() {
                            var refundButton = $(".button.refund-items");
                            refundButton.prop("disabled", true);
                            refundButton.html('');
                        });                
                    })(jQuery);
                </script>
                <?php
            }        
        
            return $actions;
        } catch (Exception $e) {
            WC_YottaPay_Common::get_logger()->warning($e->getMessage(), ['source' => WC_YottaPay_Common::WC_LOG_FILENAME]);
            return $actions;
        }
    }
}
